﻿package info.kazushi {
	import flash.events.EventDispatcher;
	import flash.system.Security;
	import flash.events.Event;
	import flash.events.DataEvent;
	import flash.events.SecurityErrorEvent;
	import flash.net.XMLSocket;
	import flash.utils.Timer;
	import flash.events.TimerEvent;
	import flash.errors.IOError;
	
	/**
	* ...
	* @author kazushi mukaiyama
	*/

	public class Client extends EventDispatcher
	{
		private var _host:String;
		private var _port:int;
		private var _name:String;
		private var _authorized:Boolean = false;
		private var _autoConnect:Boolean = false;
		private var _delay:int = 1000;
		private var _retryCount:int = 3;
		
		private var xmlsock:XMLSocket;
		private var timer:Timer;
		
		public function Client(n:String) 
		{
			_name = n;
			timer = new Timer(_delay, _retryCount);
			timer.addEventListener(TimerEvent.TIMER, onTick);
			timer.addEventListener(TimerEvent.TIMER_COMPLETE, onComplete);
			
			xmlsock = new XMLSocket();
			xmlsock.addEventListener(SecurityErrorEvent.SECURITY_ERROR, onSecurityError);
			xmlsock.addEventListener(DataEvent.DATA, onServerData);
			xmlsock.addEventListener(Event.CONNECT, onConnect);
			xmlsock.addEventListener(Event.CLOSE, onClose);
			Security.loadPolicyFile("xmlsocket://" + _host + ":" + _port);
		}

		public function get host():String { return _host; }	
		public function set host(value:String):void 
		{
			_host = value;
		}
		
		public function get port():int { return _port; }	
		public function set port(value:int):void 
		{
			_port = value;
		}
		
		public function get name():String { return _name; }	
		public function set name(value:String):void 
		{
			_name = value;
		}
		
		public function get authorized():Boolean { return _authorized; }
		public function set authorized(value:Boolean):void 
		{
			_authorized = value;
		}
		
		public function get autoConnect():Boolean { return _autoConnect; }
		public function set autoConnect(value:Boolean):void 
		{
			_autoConnect = value;
		}
		
		public function get retryCount():int { return _retryCount; }
		public function set retryCount(value:int):void 
		{
			_retryCount = value;
			timer.repeatCount = _retryCount;
		}
		
		public function login():void {
			timer.reset();
			timer.start();
		}
		
		public function logout():void {
			if (_authorized) {
				// disconnect
			}
		}
		
		public function send(data:*):void {
			if (_authorized) {
				try{
					xmlsock.send(data);
				} catch (e:IOError)
				{
					dispatch(ClientEvent.ERROR, "data sending error");
				}
			}
		}
		
		private function onConnect(event:Event):void
		{
			if (!_authorized) //ポリシーファイルを手に入れて認証がまだの2度目のコネクトのとき
			{
				var helo:XML = <client name={_name} command='helo' />;
				xmlsock.send(helo.toXMLString()); //整形のため
				timer.reset();
			}
		}
		
		private function onClose(event:Event):void
		{
			dispatch(ClientEvent.ERROR, "connection closed. reconnecting...");
			_authorized = false; //切断されると必ず認証フラグをリセット
			timer.start(); //自動再接続
		}
		
		private function onTick(event:TimerEvent):void
		{
			try 
			{
				xmlsock.connect(_host, _port);
			} catch (e:*)
			{
				// エラーを無視
			}
	}
		
		private function onComplete(event:TimerEvent):void
		{
			dispatch(ClientEvent.ERROR, "timeout");
		}
		
		private function onServerData(event:DataEvent):void
		{
			var data:XML = new XML(event.data);
			var recievedName:String = data.@name;
			var recievedCommand:String = data.@command;
			
			if (data.name()=="host" && recievedName==_name) {
				//認証返答
				if (recievedCommand == "hola") {
					_authorized = true;
					dispatch(ClientEvent.AUTHORIZED, event.data);
				}
			} else {
				//ここで音響からの命令をなんかする。
				dispatch(ClientEvent.RECIEVED, event.data);
			}
		}
		
		private function onSecurityError(event:SecurityErrorEvent):void
		{
			dispatch(ClientEvent.ERROR, "security");
			login(); //自動再接続
		}
		
		private function dispatch(type:String, data:String){
			var e:ClientEvent = new ClientEvent(type);
			e.data = data;
			dispatchEvent(e);
		}
	}
}