package info.kazushi.net.server.clientDispacher;

/**
 * Class: ClientDispatcher
 * @author: made     by Kazushi Mukaiyama <i@kazushi.info>    2008/05/05
 *
 */

import info.kazushi.net.server.XMLServer;
import info.kazushi.net.server.Protocol;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.StringReader;
import java.net.Socket;
import java.util.Calendar;
import java.util.Random;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;


public class ClientDispatcher extends Thread {
	public int id;
	public String name;
	public String address;
	public boolean authorized;

	public Socket socket;
	private BufferedReader b_in;
	private PrintWriter b_out;
	private XMLServer server;

	private boolean isStopping = false;

	static Random generator = new Random();

	private Cue cue;
	private CueSetter cueSetter;
	private CueGetter cueGetter;

	public ClientDispatcher(Socket s, int id, XMLServer server) {
		super("ClientDispatcher");

		cue = new Cue();
		cueSetter = new CueSetter(cue, this);
		cueGetter = new CueGetter(cue, this);

		socket = s;
		this.id = id;
		this.server = server;

		authorized = false;

		try {
			// create streams for client
			b_in = new BufferedReader(new InputStreamReader(socket
					.getInputStream(), Protocol.ENCODING));
			b_out = new PrintWriter(new OutputStreamWriter(socket
					.getOutputStream(), Protocol.ENCODING), true);
			cueSetter.start();
			this.start();
		} catch (IOException e) {
			if (server.getVisibleGUI()) {
				server.systemMessage.append("Connection Error: " + e + "\n");
			} else {
				System.out.print("Connection Error: " + e + "\n");
			}
			// e.printStackTrace();
			if (this.isAlive())
				this.setStopping(true);
			XMLServer.clients.remove(new Integer(this.id));
			try {
				socket.close();
			} catch (IOException ie) {
			}
		}

	}

	public void run() {
		while (true) {
			checkClient();
			yield();
			if (isStopping) {
				break;
			}
		}
	}

	public boolean getStopping() {
		return isStopping;
	}

	public void setStopping(boolean b) {
		isStopping = b;
	}

	private void checkClient() {
		String data = null;

		data = cueGetter.get();
		//System.out.println(line);

		Document document = _xmlRead(data);
		if(document==null) return;
		
		Node node = document.getFirstChild();
		
		if ((node.getNodeName()).equals("client"))
		{
			String command = null;
	        NamedNodeMap attrs = node.getAttributes();  // NamedNodeMap̎擾
	        if (attrs!=null) {
	        	Node attr = attrs.getNamedItem("command");
	        	command = attr.getNodeValue();
	 		}
			if (!authorized) { // authorization at first
				//NCAgHELO󂯎c
				if (command.equals("helo"))
				{
			        if (attrs!=null) {
			        	Node attr = attrs.getNamedItem("name");
			        	name = attr.getNodeValue();
			 		}
					address = socket.getInetAddress().getHostAddress();
					authorized = true; // approved!
					
					String buf = "<host name='"+name+"' command='hola' />";
					send(buf);
				}
			} else {
				//F،͂̂܂܃NCAgɊۓBigj
				server.sendDataToClients(data, this.id);
			}
		} else {
			System.out.println("this is not a <client>.\n");
		}

	}

	private synchronized void disconnect() {
		cueSetter.setStopping(true);
		cue.set("don't wait!"); // dummy to notify for cueGetter.get()
		this.setStopping(true);

		server.disconnect(this);

		authorized = false; // disconnected!
	}

	String recieve() {
		//NCAg̃f[^󂯎B'\0'I[ł邱ƂɒӁI
		String str = "";
		
		try {
	        // ꕶǂݍށiFlash]܂łŎ~܂
	        int c = b_in.read();
	        // G[iʐM؂ꂽ
	        if(c == -1){
				if (server.getVisibleGUI()) {
					server.systemMessage
							.append("Recieve null: ID=" + id + "\n");
				} else {
					System.out.print("Recieve null: ID=" + id + "\n");
				}
				disconnect();
	        }
	        // '\0'܂ŌJԂ
	        while (c != '\0'){
	          str += (char)c;	// Ɍ
	          c = b_in.read();	// 1ǂݍ
	        }
	        
	        // |V[t@C̗v
	        if(str.startsWith("<policy-file-request/>")){
	        	System.out.println("policy!");
	        	// |V[t@CXML`œ]
	        	String policy = "<cross-domain-policy><allow-access-from domain="
        			+ '"' + "*" + '"' + " to-ports=" + '"' + "*" + '"'
        			+ "/></cross-domain-policy>\0";
	        	b_out.write(policy);
	    		b_out.flush();
	    		if (server.getVisibleGUI() && server.sendLog.isShowing()) {
	    			server.sendLog.append(policy + ": to "
	    					+ socket.getInetAddress().getHostName() + ": time "
	    					+ Calendar.getInstance().getTime().toString() + "\n");
	    		}
	    		// f[^𑗂ƒɐؒf
	    		disconnect();
	        }

	        
		} catch (IOException e) {
			if (server.getVisibleGUI()) {
				server.systemMessage.append("Recieve Error: " + e + ": ID="
						+ id + "\n");
			} else {
				System.out.print("Recieve Error: " + e + ": ID=" + id + "\n");
			}
			disconnect();
		}

		if (server.getVisibleGUI() && server.recieveLog.isShowing()) {
			server.recieveLog.append(str + ": from "
					+ socket.getInetAddress().getHostName() + ": time "
					+ Calendar.getInstance().getTime().toString() + "\n");
		}
		return str;
	}

	public void send(String str) {
		//NCAgɃf[^𑗂
		b_out.write(str + "\0");
		b_out.flush();
		if (server.getVisibleGUI() && server.sendLog.isShowing()) {
			server.sendLog.append(str + ": to "
					+ socket.getInetAddress().getHostName() + ": time "
					+ Calendar.getInstance().getTime().toString() + "\n");
		}
	}

	private Document _xmlRead(String str)
	{
		Document document = null;
	
		InputSource in = new InputSource(new StringReader(str));
		try
		{
			DocumentBuilder docBuilder =
				DocumentBuilderFactory.newInstance().newDocumentBuilder();
			if (in != null)
			{
				document = docBuilder.parse(in);
			}
		} catch (IOException ioe)
		{
			ioe.printStackTrace();
			return null;
		} catch (SAXParseException saxpe)
		{
			System.out.println("\nXML error in line" + saxpe.getLineNumber());
			System.out.println(saxpe.getMessage());
			//if(errMap.getLoaded())
			//{
			//	System.out.println(errMap.getErrMessage(saxpe.getMessage()));
			//}
			return null;
		} catch (ParserConfigurationException pce)
		{
			pce.printStackTrace();
			return null;
		} catch (SAXException saxe)
		{
			System.out.println(saxe.getMessage());
			return null;
		}
	
		if (document != null)
		{
			document.normalize();
		}
	
		return document;
	}
}
